---- #########################################################################
---- #                                                                       #
---- # Copyright (C) OpenTX                                                  #
-----#                                                                       #
---- # License GPLv2: http://www.gnu.org/licenses/gpl-2.0.html               #
---- #                                                                       #
---- # This program is free software; you can redistribute it and/or modify  #
---- # it under the terms of the GNU General Public License version 2 as     #
---- # published by the Free Software Foundation.                            #
---- #                                                                       #
---- # This program is distributed in the hope that it will be useful        #
---- # but WITHOUT ANY WARRANTY; without even the implied warranty of        #
---- # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
---- # GNU General Public License for more details.                          #
---- #                                                                       #
---- #########################################################################
local VALUE = 0
local COMBO = 1

local edit = false
local page = 1
local current = 1
local refreshState = 0
local refreshIndex = 0
local pageOffset = 0
local pages = {}
local fields = {}
local modifications = {}

local configFields = {
  {"Type d'Aile:", COMBO, 0x80, nil, { "Normal", "Delta", "VTail" } },
  {"Type Montage:", COMBO, 0x81, nil, { "Horz", "Horz rev.", "Vert", "Vert rev." } },
}

local settingsFields = {
  {"Fonctions SxR:", COMBO, 0x9C, nil, { "Des.", "Act." } },
  {"Quick Mode:", COMBO, 0xAA, nil, { "Des.", "Act." } },
  {"Mode CH5:", COMBO, 0xA8, nil, { "AIL2", "AUX1" } },
  {"Mode CH6:", COMBO, 0xA9, nil, { "ELE2", "AUX2" } },
  {"Direction AIL:", COMBO, 0x82, nil, { "Norm", "Inv" }, { 255, 0 } },
  {"Direction PRF:", COMBO, 0x83, nil, { "Norm", "Inv" }, { 255, 0 } },
  {"Direction DIR:", COMBO, 0x84, nil, { "Norm", "Inv" }, { 255, 0 } },
  {"Direction AIL2:", COMBO, 0x9A, nil, { "Norm", "Inv" }, { 255, 0 } },
  {"Direction PRF2:", COMBO, 0x9B, nil, { "Norm", "Inv" }, { 255, 0 } },
  {"Gain stabili. AIL:", VALUE, 0x85, nil, 0, 200, "%"},
  {"Gain stabili. PRF:", VALUE, 0x86, nil, 0, 200, "%"},
  {"Gain stabili. DIR:", VALUE, 0x87, nil, 0, 200, "%"},
  {"Gain auto level AIL:", VALUE, 0x88, nil, 0, 200, "%"},
  {"Gain auto level PRF:", VALUE, 0x89, nil, 0, 200, "%"},
  {"Gain vertical PRF:", VALUE, 0x8C, nil, 0, 200, "%"},
  {"Gain vertical DIR:", VALUE, 0x8D, nil, 0, 200, "%"},
  {"Gain de lacet AIL:", VALUE, 0x8E, nil, 0, 200, "%"},
  {"Gain de lacet DIR:", VALUE, 0x90, nil, 0, 200, "%"},
  {"Off. auto angle AIL:", VALUE, 0x91, nil, -20, 20, "%", 0x6C},
  {"Off. auto angle PRF:", VALUE, 0x92, nil, -20, 20, "%", 0x6C},
  {"Off. angle vert. PRF:", VALUE, 0x95, nil, -20, 20, "%", 0x6C},
  {"Off. angle vert. DIR:", VALUE, 0x96, nil, -20, 20, "%", 0x6C},
  {"Off. angle lacet AIL:", VALUE, 0x97, nil, -20, 20, "%", 0x6C},
  {"Off. angle lacet DIR:", VALUE, 0x99, nil, -20, 20, "%", 0x6C},
}

-- Change display attribute to current field
local function addField(step)
  local field = fields[current]
  local min, max
  if field[2] == VALUE then
    min = field[5]
    max = field[6]
  elseif field[2] == COMBO then
    min = 0
    max = #(field[5]) - 1
  end
  if (step < 0 and field[4] > min) or (step > 0 and field[4] < max) then
    field[4] = field[4] + step
  end
end

-- Select the next or previous page
local function selectPage(step)
  page = 1 + ((page + step - 1 + #pages) % #pages)
  refreshIndex = 0
  pageOffset = 0
end

-- Select the next or previous editable field
local function selectField(step)
  current = 1 + ((current + step - 1 + #fields) % #fields)
  if current > 7 + pageOffset then
    pageOffset = current - 7
  elseif current <= pageOffset then
    pageOffset = current - 1
  end
end

local function drawProgressBar()
  local width = (70 * refreshIndex) / #fields
  lcd.drawRectangle(30, 1, 70, 6)
  lcd.drawFilledRectangle(32, 3, width, 2);
end

-- Redraw the current page
local function redrawFieldsPage()
  lcd.clear()
  lcd.drawScreenTitle("SxR", page, #pages)

  if refreshIndex < #fields then
    drawProgressBar()
  end

  for index = 1, 7, 1 do
    local field = fields[pageOffset+index]
    if field == nil then
      break
    end

    local attr = current == (pageOffset+index) and ((edit == true and BLINK or 0) + INVERS) or 0

    lcd.drawText(0, 1+8*index, field[1])

    if field[4] == nil then
      lcd.drawText(LCD_W, 1+8*index, "---", attr + RIGHT)
    else
      if field[2] == VALUE then
        lcd.drawNumber(LCD_W, 1+8*index, field[4], attr + RIGHT)
      elseif field[2] == COMBO then
        if field[4] >= 0 and field[4] < #(field[5]) then
          lcd.drawText(LCD_W, 1+8*index, field[5][1+field[4]], attr + RIGHT)
        end
      end
    end
  end
end

local function telemetryRead(field)
  return sportTelemetryPush(0x17, 0x30, 0x0C30, field)
end

local function telemetryWrite(field, value)
  return sportTelemetryPush(0x17, 0x31, 0x0C30, field + value*256)
end

local telemetryPopTimeout = 0
local function refreshNext()
  if refreshState == 0 then
    if #modifications > 0 then
      telemetryWrite(modifications[1][1], modifications[1][2])
      modifications[1] = nil
    elseif refreshIndex < #fields then
      local field = fields[refreshIndex + 1]
      if telemetryRead(field[3]) == true then
        refreshState = 1
        telemetryPopTimeout = getTime() + 120 -- normal delay is 500ms
      end
    end
  elseif refreshState == 1 then
    local physicalId, primId, dataId, value = sportTelemetryPop()
    if physicalId == 0x1A and primId == 0x32 and dataId == 0x0C30 then
      local fieldId = value % 256
      local field = fields[refreshIndex + 1]
      if fieldId == field[3] then
        local value = math.floor(value / 256)
        if field[3] == 0xAA then
          value = bit32.band(value, 0x0001)
        end
        if field[3] >= 0x9E and field[3] <= 0xA0 then
          local b1 = value % 256
          local b2 = math.floor(value / 256)
          value = b1*256 + b2
          value = value - bit32.band(value, 0x8000) * 2
        end
        if field[2] == COMBO and #field == 6 then
          for index = 1, #(field[6]), 1 do
            if value == field[6][index] then
              value = index - 1
              break
            end
          end
        elseif field[2] == VALUE and #field == 8 then
          value = value - field[8] + field[5]
        end
        fields[refreshIndex + 1][4] = value
        refreshIndex = refreshIndex + 1
        refreshState = 0
      end
    elseif getTime() > telemetryPopTimeout then
      fields[refreshIndex + 1][4] = nil
      refreshIndex = refreshIndex + 1
      refreshState = 0
    end
  end
end

local function updateField(field)
  local value = field[4]
  if field[2] == COMBO and #field == 6 then
    value = field[6][1+value]
  elseif field[2] == VALUE and #field == 8 then
    value = value + field[8] - field[5]
  end
  modifications[#modifications+1] = { field[3], value }
end

-- Main
local function runFieldsPage(event)
  if event == EVT_EXIT_BREAK then -- exit script
    return 2
  elseif event == EVT_ENTER_BREAK then -- toggle editing/selecting current field
    if fields[current][4] ~= nil then
      edit = not edit
      if edit == false then
        updateField(fields[current])
      end
    end
  elseif edit then
    if event == EVT_PLUS_FIRST or event == EVT_ROT_RIGHT or event == EVT_PLUS_REPT then
      addField(1)
    elseif event == EVT_MINUS_FIRST or event == EVT_ROT_LEFT or event == EVT_MINUS_REPT then
      addField(-1)
    end
  else
    if event == EVT_MINUS_FIRST or event == EVT_ROT_RIGHT then
      selectField(1)
    elseif event == EVT_PLUS_FIRST or event == EVT_ROT_LEFT then
      selectField(-1)
    end
  end
  redrawFieldsPage()
  return 0
end

local mountText = { "Etiquette vers le haut", "Etiquette vers le bas", "Etiquette a gauche quand", "Etiquette a droite quand" }

local function runConfigPage(event)
  fields = configFields
  local result = runFieldsPage(event)
  if fields[2][4] ~= nil then
    lcd.drawText(1, 30, "Pins vers la queue")
    lcd.drawText(1, 40, mountText[1 + fields[2][4]])
    if fields[2][4] > 1 then
      lcd.drawText(1, 50, "on regarde depuis la queue")
    end
  end
  return result
end

local function runSettingsPage(event)
  fields = settingsFields
  return runFieldsPage(event)
end

-- Init
local function init()
  current, edit, refreshState, refreshIndex = 1, false, 0, 0
  pages = {
    runConfigPage,
    runSettingsPage,
  }
end

-- Main
local function run(event)
  if event == nil then
    error("Ce n'est pas un script modele !!")
    return 2
  elseif event == EVT_PAGE_BREAK then
    selectPage(1)
  elseif event == EVT_PAGE_LONG then
    killEvents(event);
    selectPage(-1)
  end

  local result = pages[page](event)
  refreshNext()

  return result
end

return { init=init, run=run }
